/// <reference path='index.d.ts' />
/*
Before All
- Create a JSX style cache
- Load and cache UMD modules for React unit testing
*/
before(() => {
  Cypress.stylesCache = new Map()
  Cypress.React = ''
  Cypress.ReactDOM = ''
  // eslint-disable no-return-assign
  cy.log('Cypress-React: Load and cache React & ReactDOM')
    .readFile('node_modules/react/umd/react.development.js')
    .then(file => Cypress.React = file)
    .readFile('node_modules/react-dom/umd/react-dom.development.js')
    .then(file => Cypress.ReactDOM = file)
})

/** Initialize an empty document w/ ReactDOM and DOM events.
    @function   cy.injectReactDOM
**/
Cypress.Commands.add('injectReactDOM', () => {
  return cy
    .log('Injecting ReactDOM for Unit Testing')
    .reload({ log: false })
    .then(() => {
      // include React and ReactDOM to force DOM to register all DOM event listeners
      // otherwise the component will NOT be able to dispatch any events
      // when it runs the second time
      // https://github.com/bahmutov/cypress-react-unit-test/issues/3
      var html = `<body>
          <div id="cypress-jsdom"></div>
          ${Cypress.Styles || ''}
          <script>${Cypress.React}</script>
          <script>${Cypress.ReactDOM}</script>
        </body>`
      const document = cy.state('document')
      document.write(html)
      document.close()
    })
})

/** Caches styles from previously compiled components for reuse
    @function   cy.copyComponentStyles
    @param      {Object}  component
**/
Cypress.Commands.add('copyComponentStyles', (component) => {
  // need to find same component when component is recompiled
  // by the JSX preprocessor. Thus have to use something else,
  // like component name
  const hash = component.type.name
  const document = cy.state('document')
  let styles = document.querySelectorAll('head style')
  if (styles.length) {
    cy.log('injected %d styles', styles.length)
    Cypress.stylesCache.set(hash, styles)
  } else {
    cy.log('No styles injected for this component, checking cache')
    if (Cypress.stylesCache.has(hash)) {
      styles = Cypress.stylesCache.get(hash)
    } else {
      styles = null
    }
  }
  if (!styles) {
    return
  }
  const parentDocument = window.parent.document
  const projectName = Cypress.config('projectName')
  const appIframeId = "Your App: '" + projectName + "'"
  const appIframe = parentDocument.getElementById(appIframeId)
  var head = appIframe.contentDocument.querySelector('head')
  styles.forEach(function (style) {
    head.appendChild(style)
  })
  return
})

/** Mount a React component in a blank document; register it as an alias
    To access: use an alias, e.g.cy.get('@Component').its('state').should(...)
    @function   cy.mount
    @param      {Object}  jsx
    @param      {string}  [Component]   alias
**/
Cypress.Commands.add('mount', (jsx, alias) => {
  // Get the displayname property via the component constructor
  const displayname = alias || jsx.type.prototype.constructor.name
  cy
    .injectReactDOM()
    .log(`ReactDOM.render(<${displayname} ... />)`, jsx.props)
    .window({ log: false })
    .then(win => {
      const { ReactDOM } = win
      const document = cy.state('document')
      const component = ReactDOM.render(jsx, document.getElementById('cypress-jsdom'))
      cy.wrap(component, { log: false }).as(displayname)
    })
  cy.copyComponentStyles(jsx)
})